Package: oligo
Version: 1.62.2
Title: Preprocessing tools for oligonucleotide arrays
Author: Benilton Carvalho and Rafael Irizarry
Contributors: Ben Bolstad, Vincent Carey, Wolfgang Huber, Harris
        Jaffee, Jim MacDonald, Matt Settles, Guido Hooiveld
Maintainer: Benilton Carvalho <benilton@unicamp.br>
Depends: R (>= 3.2.0), BiocGenerics (>= 0.13.11), oligoClasses (>=
        1.29.6), Biobase (>= 2.27.3), Biostrings (>= 2.35.12)
Imports: affyio (>= 1.35.0), affxparser (>= 1.39.4), DBI (>= 0.3.1),
        ff, graphics, methods, preprocessCore (>= 1.29.0), RSQLite (>=
        1.0.0), splines, stats, stats4, utils, zlibbioc
Enhances: doMC, doMPI
LinkingTo: preprocessCore
Suggests: BSgenome.Hsapiens.UCSC.hg18, hapmap100kxba, pd.hg.u95av2,
        pd.mapping50k.xba240, pd.huex.1.0.st.v2, pd.hg18.60mer.expr,
        pd.hugene.1.0.st.v1, maqcExpression4plex, genefilter, limma,
        RColorBrewer, oligoData, BiocStyle, knitr, RUnit, biomaRt,
        AnnotationDbi, ACME, RCurl
VignetteBuilder: knitr
Description: A package to analyze oligonucleotide arrays
        (expression/SNP/tiling/exon) at probe-level. It currently
        supports Affymetrix (CEL files) and NimbleGen arrays (XYS
        files).
License: LGPL (>= 2)
Collate: AllGenerics.R methods-GenericArrays.R methods-GeneFeatureSet.R
        methods-ExonFeatureSet.R methods-ExpressionFeatureSet.R
        methods-ExpressionSet.R methods-LDS.R methods-FeatureSet.R
        methods-SnpFeatureSet.R methods-SnpCnvFeatureSet.R
        methods-TilingFeatureSet.R methods-HtaFeatureSet.R
        methods-DBPDInfo.R methods-background.R methods-normalization.R
        methods-summarization.R read.celfiles.R read.xysfiles.R
        utils-general.R utils-selectors.R todo-snp.R functions-crlmm.R
        functions-snprma.R justSNPRMA.R justCRLMM.R methods-snp6.R
        methods-genotype.R methods-PLMset.R zzz.R
LazyLoad: Yes
biocViews: Microarray, OneChannel, TwoChannel, Preprocessing, SNP,
        DifferentialExpression, ExonArray, GeneExpression, DataImport
git_url: https://git.bioconductor.org/packages/oligo
git_branch: RELEASE_3_16
git_last_commit: 05a7e3c
git_last_commit_date: 2023-01-19
Date/Publication: 2023-01-19
NeedsCompilation: yes
Packaged: 2023-01-19 22:17:44 UTC; biocbuild
